from __future__ import print_function, division
import scs
import numpy as np
from scipy import sparse
import gen_random_cone_prob as tools

#############################################
#  Uses scs to solve a random cone problem  #
#############################################


def import_error(msg):
    print()
    print("## IMPORT ERROR:" + msg)
    print()


try:
    import pytest
except ImportError:
    import_error("Please install pytest to run tests.")
    raise

np.random.seed(1)

# cone:
K = {
    "z": 10,
    "l": 15,
    "q": [5, 10, 0, 1],
    "s": [3, 4, 0, 0, 1, 10],
    "ep": 10,
    "ed": 10,
    "p": [-0.25, 0.5, 0.75, -0.33],
}
m = tools.get_scs_cone_dims(K)
params = {"verbose": True, "eps_abs": 1e-5, "eps_rel": 1e-5, "eps_infeas": 1e-5}


try:
    import _scs_mkl

    def test_solve_feasible():
        data, p_star = tools.gen_feasible(K, n=m // 3, density=0.1)
        solver = scs.SCS(data, K, mkl=True, **params)
        sol = solver.solve()
        x = sol["x"]
        y = sol["y"]
        s = sol["s"]
        np.testing.assert_almost_equal(np.dot(data["c"], x), p_star, decimal=3)
        np.testing.assert_almost_equal(np.dot(data["c"], x), p_star, decimal=3)
        np.testing.assert_array_less(
            np.linalg.norm(data["A"] @ x - data["b"] + s), 1e-3
        )
        np.testing.assert_array_less(
            np.linalg.norm(data["A"].T @ y + data["c"]), 1e-3
        )
        np.testing.assert_almost_equal(s.T @ y, 0.0)
        np.testing.assert_almost_equal(s, tools.proj_cone(s, K), decimal=4)
        np.testing.assert_almost_equal(y, tools.proj_dual_cone(y, K), decimal=4)

    def test_solve_infeasible():
        data = tools.gen_infeasible(K, n=m // 2)
        solver = scs.SCS(data, K, mkl=True, **params)
        sol = solver.solve()
        y = sol["y"]
        np.testing.assert_array_less(np.linalg.norm(data["A"].T @ y), 1e-3)
        np.testing.assert_array_less(data["b"].T @ y, -0.1)
        np.testing.assert_almost_equal(y, tools.proj_dual_cone(y, K), decimal=4)

    def test_solve_unbounded():
        data = tools.gen_unbounded(K, n=m // 2)
        solver = scs.SCS(data, K, mkl=True, **params)
        sol = solver.solve()
        x = sol["x"]
        s = sol["s"]
        np.testing.assert_array_less(np.linalg.norm(data["A"] @ x + s), 1e-3)
        np.testing.assert_array_less(data["c"].T @ x, -0.1)
        np.testing.assert_almost_equal(s, tools.proj_cone(s, K), decimal=4)

except ImportError:
    import_error("Skipping MKL tests as SCS MKL is not installed.")



from __future__ import print_function, division
import scs
import numpy as np
from scipy import sparse, randn
import gen_random_cone_prob as tools

#############################################
#  Uses scs to solve a random cone problem  #
#############################################


def main():
    flags = [(False, False), (True, False)]
    try:
        import _scs_gpu

        flags += [(True, True)]
    except ImportError:
        pass

    for use_indirect, gpu in flags:
        np.random.seed(1)
        solve_feasible(use_indirect, gpu)
        solve_infeasible(use_indirect, gpu)
        solve_unbounded(use_indirect, gpu)


def solve_feasible(use_indirect, gpu):
    # cone:
    K = {
        "f": 10,
        "l": 15,
        "q": [5, 10, 0, 1],
        "s": [3, 4, 0, 0, 1],
        "ep": 10,
        "ed": 10,
        "p": [-0.25, 0.5, 0.75, -0.33],
    }
    m = tools.get_scs_cone_dims(K)
    data, p_star = tools.gen_feasible(K, n=m // 3, density=0.01)
    params = {"normalize": True, "scale": 5}

    sol = scs.solve(data, K, use_indirect=use_indirect, gpu=gpu, **params)
    x = sol["x"]
    y = sol["y"]
    print("p*  = ", p_star)
    print("pri error = ", (np.dot(data["c"], x) - p_star) / p_star)
    print("dual error = ", (-np.dot(data["b"], y) - p_star) / p_star)


def solve_infeasible(use_indirect, gpu):
    K = {
        "f": 10,
        "l": 15,
        "q": [5, 10, 0, 1],
        "s": [3, 4, 0, 0, 1],
        "ep": 10,
        "ed": 10,
        "p": [-0.25, 0.5, 0.75, -0.33],
    }
    m = tools.get_scs_cone_dims(K)
    data = tools.gen_infeasible(K, n=m // 3)
    params = {"normalize": True, "scale": 0.5}
    sol = scs.solve(data, K, use_indirect=use_indirect, gpu=gpu, **params)


def solve_unbounded(use_indirect, gpu):
    K = {
        "f": 10,
        "l": 15,
        "q": [5, 10, 0, 1],
        "s": [3, 4, 0, 0, 1],
        "ep": 10,
        "ed": 10,
        "p": [-0.25, 0.5, 0.75, -0.33],
    }
    m = tools.get_scs_cone_dims(K)
    data = tools.gen_unbounded(K, n=m // 3)
    params = {"normalize": True, "scale": 0.5}
    sol = scs.solve(data, K, use_indirect=use_indirect, gpu=gpu, **params)


if __name__ == "__main__":
    main()



from __future__ import print_function
import platform

## import utilities to generate random cone probs:
import sys
import gen_random_cone_prob as tools


def import_error(msg):
    print()
    print("## IMPORT ERROR:" + msg)
    print()


try:
    import pytest
except ImportError:
    import_error("Please install pytest to run tests.")
    raise

try:
    import scs
except ImportError:
    import_error("You must install the scs module before running tests.")
    raise

try:
    import numpy as np
    from numpy.testing import assert_almost_equal
except ImportError:
    import_error("Please install numpy.")
    raise

try:
    import scipy.sparse as sp
except ImportError:
    import_error("Please install scipy.")
    raise


def assert_(str1, str2):
    if str1 != str2:
        print("assert failure: %s != %s" % (str1, str2))
    assert str1 == str2


def check_infeasible(sol):
    assert_(sol["info"]["status"], "infeasible")


def check_unbounded(sol):
    assert_(sol["info"]["status"], "unbounded")


np.random.seed(0)
num_feas = 50
num_unb = 10
num_infeas = 10

opts = {
    "max_iters": 10000,
    "eps_abs": 1e-5,
    "eps_infeas": 1e-5,
}
K = {
    "z": 10,
    "l": 25,
    "q": [5, 10, 0, 1],
    "s": [2, 1, 2, 0, 1, 10, 8],
    "ep": 0,
    "ed": 0,
    "p": [0.25, -0.75, 0.33, -0.33, 0.2],
}
m = tools.get_scs_cone_dims(K)


@pytest.mark.parametrize("use_indirect", [False, True])
def test_feasible(use_indirect):
    for i in range(num_feas):
        data, p_star = tools.gen_feasible(K, n=m // 3, density=0.1)
        solver = scs.SCS(data, K, use_indirect=use_indirect, **opts)
        sol = solver.solve()
        assert_almost_equal(np.dot(data["c"], sol["x"]), p_star, decimal=2)
        assert_almost_equal(np.dot(-data["b"], sol["y"]), p_star, decimal=2)


@pytest.mark.parametrize("use_indirect", [False, True])
def test_infeasible(use_indirect):
    for i in range(num_infeas):
        data = tools.gen_infeasible(K, n=m // 2)
        solver = scs.SCS(data, K, use_indirect=use_indirect, **opts)
        sol = solver.solve()
        check_infeasible(sol)


# TODO: indirect solver has trouble in this test, so disable for now
@pytest.mark.parametrize("use_indirect", [False])
def test_unbounded(use_indirect):
    for i in range(num_unb):
        data = tools.gen_unbounded(K, n=m // 2)
        solver = scs.SCS(data, K, use_indirect=use_indirect, **opts)
        sol = solver.solve()
        check_unbounded(sol)



from __future__ import print_function, division
import scs
import numpy as np
from scipy import sparse
import gen_random_cone_prob as tools

#############################################
#  Uses scs to solve a random cone problem  #
#############################################


def import_error(msg):
    print()
    print("## IMPORT ERROR:" + msg)
    print()


try:
    import pytest
except ImportError:
    import_error("Please install pytest to run tests.")
    raise

flags = [(False, False), (True, False)]
try:
    import _scs_gpu

    flags += [(True, True)]
except ImportError:
    pass

np.random.seed(1)

# cone:
K = {
    "z": 10,
    "l": 15,
    "q": [5, 10, 0, 1],
    "s": [3, 4, 0, 0, 1, 10],
    "ep": 10,
    "ed": 10,
    "p": [-0.25, 0.5, 0.75, -0.33],
}
m = tools.get_scs_cone_dims(K)
params = {"verbose": True, "eps_abs": 1e-5, "eps_rel": 1e-5, "eps_infeas": 1e-5}


@pytest.mark.parametrize("use_indirect,gpu", flags)
def test_solve_feasible(use_indirect, gpu):
    data, p_star = tools.gen_feasible(K, n=m // 3, density=0.1)
    solver = scs.SCS(data, K, use_indirect=use_indirect, gpu=gpu, **params)
    sol = solver.solve()
    x = sol["x"]
    y = sol["y"]
    s = sol["s"]
    np.testing.assert_almost_equal(np.dot(data["c"], x), p_star, decimal=3)
    np.testing.assert_almost_equal(np.dot(data["c"], x), p_star, decimal=3)
    np.testing.assert_array_less(
        np.linalg.norm(data["A"] @ x - data["b"] + s), 1e-3
    )
    np.testing.assert_array_less(
        np.linalg.norm(data["A"].T @ y + data["c"]), 1e-3
    )
    np.testing.assert_almost_equal(s.T @ y, 0.0)
    np.testing.assert_almost_equal(s, tools.proj_cone(s, K), decimal=4)
    np.testing.assert_almost_equal(y, tools.proj_dual_cone(y, K), decimal=4)


@pytest.mark.parametrize("use_indirect,gpu", flags)
def test_solve_infeasible(use_indirect, gpu):
    data = tools.gen_infeasible(K, n=m // 2)
    solver = scs.SCS(data, K, use_indirect=use_indirect, gpu=gpu, **params)
    sol = solver.solve()
    y = sol["y"]
    np.testing.assert_array_less(np.linalg.norm(data["A"].T @ y), 1e-3)
    np.testing.assert_array_less(data["b"].T @ y, -0.1)
    np.testing.assert_almost_equal(y, tools.proj_dual_cone(y, K), decimal=4)


# TODO: indirect solver has trouble in this test, so disable for now
@pytest.mark.parametrize("use_indirect,gpu", [(False, False)])
def test_solve_unbounded(use_indirect, gpu):
    data = tools.gen_unbounded(K, n=m // 2)
    solver = scs.SCS(data, K, use_indirect=use_indirect, gpu=gpu, **params)
    sol = solver.solve()
    x = sol["x"]
    s = sol["s"]
    np.testing.assert_array_less(np.linalg.norm(data["A"] @ x + s), 1e-3)
    np.testing.assert_array_less(data["c"].T @ x, -0.1)
    np.testing.assert_almost_equal(s, tools.proj_cone(s, K), decimal=4)
